/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2013                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#ifndef _NEW_DERIVATED_METRIC_WIDGET_H
#define _NEW_DERIVATED_METRIC_WIDGET_H


#include <QDialog>
#include <QLineEdit>
#include <QTextEdit>
#include <QComboBox>
#include <QString>
#include <QLabel>
#include <QDropEvent>
#include <QDragEnterEvent>
#include <QTextStream>
#include <QKeyEvent>
#include "Cube.h"
#include "CubeMetric.h"
#include "CubePL1Driver.h"
#include "CubePLSyntaxHighlighter.h"
#include "HelpBrowser.h"

/*-------------------------------------------------------------------------*/
/**
 * @file  NewDerivatedMetricWidget.h
 * @brief Declaration of the widget, used to create a new derivated metric
 *
 * This header file provides the declaration of the class HelpBrowser.
 */
/*-------------------------------------------------------------------------*/

#define HTML_DOC_CUBEPL "file://" CUBE_PREFIX "/share/doc/cube/cubepl.html"
#define HTML_DOC_DERIVED_METRICS "file://" CUBE_PREFIX "/share/doc/cube/cube-derived-metrics.html"
#define HTML_DOC_DERIVED_METRIC_FROM_FILE "file://" CUBE_PREFIX "/share/doc/cube/derived-metric-from-file.html"

/*-------------------------------------------------------------------------*/
/**
 * @class HelpBrowser
 * @brief Provides a simple dialog window for displaying HTML help texts
 *
 * This class provides a simple non-modal dialog window for displaying
 * HTML help texts. The displayed text (including referenced image data)
 * will be cached until either a new document is loaded, in which case its
 * data will be cached, or the instance is destroyed (closing the dialog
 * will only hide it).
 */
/*-------------------------------------------------------------------------*/

class NewDerivatedMetricWidget : public QDialog
{
    Q_OBJECT

public:
    /// @name Constructors &amp; destructor
    /// @{

    NewDerivatedMetricWidget( cube::Cube*   cube,
                              cube::Metric* met_parent = 0,
                              QWidget*      parent = 0 );

    /// @}
    ~NewDerivatedMetricWidget()
    {
        delete error_highlighter;
    };

public slots:
    /// @name Different slots to set up the different fields of metric:
    /// @{

    void
    setMetricType( int );

    void
    setDisplayName( const QString& );
    void
    setUniqName( const QString& );

//     void setDType(QString&);
    void
    setUom( const QString& );
    void
    setValue( const QString& );
    void
    setURL( const QString& );
    void
    setDescription( const QString& );
    void
    setCubePLExpression();
    void
    setCubePLInitExpression();

    void
    showCubePLHelp();

    void
    showKindsOfDerivedMetricsHelp();

    void
    showAvailableMetrics();

    void
    createMetricFromFile();

    void
    createMetricFromClipboard();


    void
    createMetric();

    cube::Metric*
    get_created_metric()
    {
        return created_metric;
    };

    void
    selectFileForFillingForm();




    /// @}

protected:
    void
    dragEnterEvent( QDragEnterEvent* event );
    void
    dropEvent( QDropEvent* event );
    void
    fillTheFormFromUrl( QUrl & );

    void
    fillTheFormFromClipboard();

    void
    fillTheForm( QTextStream & );

private:

    bool               calculation_ok;
    bool               calculation_init_ok;

    cube::Cube*        cube;
    cube::Metric*      parent_metric;
    cube::Metric*      created_metric;
    cube::TypeOfMetric metric_type;

    std::string        uniq_name;
    std::string        disp_name;
//     std::string dtype;
    std::string        uom;
    std::string        val;
    std::string        url;
    std::string        kind;
    std::string        descr;
    std::string        calculation;
    std::string        init_calculation;


    QComboBox*   metric_type_selection;
    QLineEdit*   display_name_input;
    QLineEdit*   unique_name_input;
//     QLineEdit * data_type_input;
    QLineEdit*   uom_input;
    QLineEdit*   value_input;
    QLineEdit*   url_input;
    QLineEdit*   description_input;
    QTextEdit*   calculation_input;
    QTextEdit*   calculation_init_input;

    QPushButton* create_metric;
    QPushButton* _whatis_help;
    QPushButton* _metric_kinds_help;
    QPushButton* metric_from_file;
    QPushButton* metric_from_clipboard;


    QLabel*                  syntax_status;
    QLabel*                  init_syntax_status;

    CubePLSyntaxHighlighter* error_highlighter;
    CubePLSyntaxHighlighter* error_init_highlighter;

    HelpBrowser*             help_browser;
};


#endif
