/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2013                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

/**
 * \file CubeCache.h
 * \brief Provides an interface for caching mechanisms in metrics
 */

#ifndef __CACHE_H
#define __CACHE_H

#include <iostream>
#include "CubeCnode.h"
#include "CubeSysres.h"
#include "CubeValues.h"
#include "CubeTypes.h"

using namespace std;

namespace cube
{
/**
 * Class "Cache" is an abstract class, which defines a interface for "Metric" to access the cached data
 */
class Cache
{
protected:
public:
    virtual
    ~Cache()
    {
    };

    virtual Value*
    getCachedValue(Cnode*, CalculationFlavour, Sysres* sysres= NULL, CalculationFlavour sf=CUBE_CALCULATE_INCLUSIVE) = 0;                             ///< Returns Value or NULL, if not present

    virtual void
    setCachedValue(Value*, Cnode*, CalculationFlavour, Sysres* sysres= NULL, CalculationFlavour sf=CUBE_CALCULATE_INCLUSIVE) = 0;                 ///< Stores or not (if not needed) a Value.

};
}

#endif
