/****************************************************************************
**  CUBE        http://www.score-p.org/                                    **
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2011-2013                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2011-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/
/**
 * 
 * this code is a facade to jtar library - Xeus Technologies 
 * http://code.google.com/p/jtar/
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 */

package scalasca.cubex.tarviewer;

import java.io.IOException;
import java.io.InputStream;

import org.xeustechnologies.jtar.TarEntry;
import org.xeustechnologies.jtar.TarInputStream;


/**

 Provides for TarInputStream a list of files in the tar file with size, offset and a name. 
 Used by Cube to get a direct access to the tar content.
*/
public class BTarInputStream extends TarInputStream{

	private long chunkCnt;
	
	public BTarInputStream(InputStream in) {
		super(in);
		chunkCnt = 0;
	}
	
	/**
	 * Gets the next entry inside the TAR archive
	 * @return BTarEntry object representing the next entry inside the TAR archive, NULL otherwise
	 * @throws IOException
	 */
	public BTarEntry next() throws IOException
	{
		TarEntry te = super.getNextEntry();
		chunkCnt++;
		if(te==null)
			return null;
		long size = te.getSize();
		
		BTarEntry bte = new BTarEntry(te.getName(), size, chunkCnt * 512, te.isDirectory());
		
		chunkCnt += size / 512 + (size%512==0?0:1);
		return bte;
	}
}
